@@ -1,4 +1,5 @@
-#' Trichotomization of ordinal test results using predictive values
+#' Function for the determination of an inconclusive interval for ordinal test scores
+#' using predictive values
 #'
 #' @export
 #' @importFrom zoo rollsum na.fill
@@ -17,12 +18,14 @@
 #'   every individual. When using a factor, please check whether the correct
 #'   order of the values is used. Further, a warning message is issued
 #'   concerning the calculation of the variance of the test when using a factor.
-#' @param reliability (no default) The known reliability of the test, used to
-#'   calculate Standard Error of Measurement (SEM). The reliability is expressed
-#'   as an applicable correlation coefficient, with values between 0 and 1. A
+#' @param reliability (default NULL) The reliability of the test, used to calculate
+#'   Standard Error of Measurement (SEM). The reliability is expressed as an
+#'   applicable correlation coefficient, with values between 0 and 1. A
 #'   Pearson's Product Moment correlation or an Intra-Class Coefficient (ICC)
-#'   will do. N.B. Setting parameter \code{roll.length} to 1 causes the
-#'   reliability of the test to be ignored.
+#'   will do. N.B. A value of 1 assumes perfect reliability, is not
+#'   realistic and prevents smoothing as it sets \code{roll.length} to 1. N.B.
+#'   Setting parameter \code{roll.length} to a value >= 1 causes the reliability
+#'   of the test to be ignored.
 #' @param pretest.prob (default = NULL) value to be used as pre-test
 #'   probability. It is used for the calculation of the post-test probabilities.
 #'   If pretest.prob = NULL, the sample prevalence is used.
@@ -31,8 +34,8 @@
 #'   NULL, it is calculated as \code{round(SEM)*2+1} (approximately a 68\%
 #'   confidence interval around the observed test score in which the true score
 #'   is expected). The roll.length needs to be uneven. When the result is even,
-#'   you need to choose a value 1 larger or smaller. When roll.length is
-#'   supplied, this is used instead of the calculated value. Furthermore,
+#'   a lower value (-1) is assigned. When roll.length is supplied, this is used
+#'   instead of the calculated value and reliability is ignored. Furthermore,
 #'   roll.length has to be >= 1 and uneven. Applicable values are 1, 3, 5, etc.
 #'   roll.length = 1 causes the reliability of the test to be ignored.
 #' @param extend (default = TRUE) The Reliable Predictive Values cannot be
@@ -43,17 +46,18 @@
 #' @param decision.odds (default = 2). The minimum odds for and against the
 #'   targeted impairment for a positive or negative classification. For the
 #'   uncertain range of test scores both the odds for and the odds against are
-#'   therefore between 2 and 1/2. A decision.odds of 1 causes all test
-#'   scores to be used for either positive or negative decisions. The limit for
-#'   the Predictive Value = decision.odds / (decision.odds+1); for a decision,
-#'   the Predictive Values needs to be larger than this limit. NB 1 Decison.odds
-#'   can be a broken number, such as .55/(1-.55), which defines the decision
-#'   limit for predictive values as .55. The default is therefore (2/3) / (1
-#'   - (2/3)) = 2. NB 2 When a test is more reliable and valid, a higher value for
-#'   decision.odds can be applied. NB 3 For serious diseases with relatively
-#'   uncomplicated cures, decision odds can be smaller than one. In that case, a
-#'   large number of false positives is unavoidable and positive decisions are
-#'   inherently uncertain. See Sonis(1999) for a discussion.
+#'   therefore between 2 and 1/2. A decision.odds of 1 causes all test scores to
+#'   be used for either positive or negative decisions. The limit for the
+#'   Predictive Value = decision.odds / (decision.odds+1); for a decision, the
+#'   Predictive Values needs to be larger than this limit. NB 1 Decision.odds can
+#'   be a broken number, such as .55/(1-.55), which defines the decision limit
+#'   for predictive values as .55. The default is therefore (2/3) / (1 - (2/3))
+#'   = 2, equal to a predictive value of .667. NB 2 When a test is more reliable
+#'   and valid, a higher value for decision.odds can be applied. NB 3 For
+#'   serious diseases with relatively uncomplicated cures, decision odds can be
+#'   smaller than one. In that case, a large number of false positives is
+#'   unavoidable and positive decisions are inherently uncertain. See
+#'   Sonis(1999) for a discussion.
 #' @param decision.use (default = 'standardized.pv'). The probability to be used
 #'   for decisions. When 'standardized.pv' is chosen, the standardized positive
 #'   predictive value is used for positive decisions and the standardized
@@ -76,7 +80,10 @@
 #'   decisions. An uncertain Interval of a single test score can be determined
 #'   with in-between values. For instance c(1.5, 0.8) defines an uncertain
 #'   interval of test score 1 for a descending ordinal test.
-#' @param digits   the number of digits used in the output.
+#' @param digits   (default = 3) the number of digits used in the output.
+#' @param use.perc (default = TRUE) Use percentages for the output. When set to
+#'   FALSE, proportions are used.
+#' @param show.table (default = FALSE) Show confusion table.
 #'
 #' @details This function can be applied to ordinal data. Uncertain test scores
 #'   are scores that have about the same density in the two distributions of
@@ -88,9 +95,9 @@
 #'
 #'   N.B. 1: Sp = Negative Decisions | true.neg.status; Se = Positive Decisions
 #'   | true.pos.status. Please note that the values for Se and Sp are
-#'   underestimated, as the uncertain test scores are considered here as errors,
-#'   which they are not. (Se and Sp are dichotomous indices.). Use
-#'   \code{\link{quality.threshold}} and
+#'   underestimated, as the uncertain test scores are considered as errors,
+#'   which they are not. (Se and Sp are intended for dichotomous thresholds.).
+#'   Use \code{\link{quality.threshold}} and
 #'   \code{\link{quality.threshold.uncertain}} for obtaining respectively
 #'   quality indices for the test scores when ignoring test scores in the
 #'   uncertain interval and the quality indices of the test scores within the
@@ -103,6 +110,10 @@
 #'   N.B. 3: Set roll.length to 1 to ignore the test reliability and obtain raw
 #'   predictive values, likelihood ratios, etc., that are not corrected for the
 #'   unreliability of the test.
+#'   
+#'   N.B. 4: In contrast to the other functions, the \code{RPV}  function can
+#'   detect more than one uncertain interval. More than one uncertain interval
+#'   almost always signals bad test quality and interpretation difficulties.
 #'
 #'   Raw predictive values compare the frequencies and provide exact sample
 #'   values and are most suitable for evaluating the sample results. When
@@ -186,13 +197,18 @@
 #' \item{rsppv: }{(more) reliable standardized positive predictive value.}
 #' \item{rilr: }{(more) reliable interval likelihood ratio.}
 #' \item{rpt.odds: }{(more) reliable posttest odds.}
-#' \item{rpt.prob: }{(more) reliable posttest probabilities.} } }
+#' \item{rpt.prob: }{(more) reliable posttest probabilities.} } 
+#' }
+#' \item{thresholds.UI: }{Numeric values for the thresholds of the uncertain
+#' interval. This is NA when there are multiple ranges of test for the uncertain
+#' interval.}
+#' \item{ranges: }{The ranges of test scores for the Negative Classifications, Uncertain, Positive
+#' Classifications.}
 #' \item{$result: }{Table of results for the current sample, calculated with the
 #' provided parameters. 
 #' \itemize{ 
 #' \item{columns: }{Negative Classifications, Uncertain, Positive
 #' Classifications.}
-#' \item{row scores: }{range of test scores for the three categories.}
 #' \item{row total.sample: }{percentage of the total sample.}
 #' \item{row correct.decisions: }{percentages of correct negative and positive
 #' decisions (NPV and PPV).}
@@ -203,7 +219,9 @@
 #' \item{row realized.odds: }{The odds that are realized in the sample for each
 #' of the three categories. NB The odds of the uncertain range of test scores
 #' concerns the odds for the targeted condition.}
-#' } } }
+#' } } 
+#' \item{$table}{Show table of counts of decisions x true status.}
+#' }
 #' @references Sonis, J. (1999). How to use and interpret interval likelihood
 #'   ratios. Family Medicine, 31, 432–437.
 #'
@@ -230,22 +248,21 @@
 #' # Positive Predictive Values. Parameter pretest.prob is set to sample prevalence.
 #' RPV(ref, test, reliability = .9, roll.length = 3)
 #' # Set roll.length = 1 to ignore test reliability (value of parameter
-#' # reliability is ignored, but must be set to some value.)
-#' RPV(ref, test, reliability = 0, roll.length = 1)
 #' # When pretest.prob is set to .5, the Post-test Probabilities are equal to
 #' # the Standardized Positive Predictive Values.
 #' RPV(ref, test, pretest.prob = .5, reliability = .9, roll.length = 3)
 #'
 
-# pretest.prob = .5; reliability=.9; roll.length = 1; extend = TRUE;
-# decision.odds = 3; decision.use = 'LR'; digits=3; preselected.thresholds=NULL
-# preselected.thresholds=c(25.9, 25.8)
+# pretest.prob = .53; reliability=.86; roll.length = 5; extend = TRUE;
+# decision.odds = 2; decision.use = 'standardized.pv'; digits=3; 
+# preselected.thresholds=c(25,22); use.perc=F
 
-RPV <- function(ref, test, pretest.prob = NULL, reliability, roll.length = NULL,
+RPV <- function(ref, test, pretest.prob = NULL, reliability = NULL, roll.length = NULL,
                 extend = TRUE, decision.odds = 2, decision.use =
                   c('standardized.pv', "posttest.probability", 'LR',
                     "predictive.value"),
-                preselected.thresholds=c(NULL, NULL), digits=3){
+                preselected.thresholds=c(NULL, NULL), digits=2,
+                use.perc = TRUE, show.table=FALSE){
 
   # require(zoo)
   df = check.data(ref,test, 'ordinal')
@@ -253,47 +270,62 @@ RPV <- function(ref, test, pretest.prob = NULL, reliability, roll.length = NULL,
   test = df$test
   decision.use = match.arg(decision.use)
 
-
   is.even <- function(x) x %% 2 == 0 # is.even(F)
   percent <- function(x, digits = digits, format = "f", ...) {
     paste0(formatC(100 * x, format = format, digits = digits, ...), "%")
   }
-  mktxtseq <- function(r) {
+  mktxtseq <- function(r) {  # r = c(3, 7,8, 10)
     d = abs(diff(c(-Inf, r)))
     w = which(d != 1)
     ranges = list()
     for (i in (1:(length(w) - 1))) {
-      ranges[i] = paste(r[w[i]], r[(w[i + 1] - 1)], sep = '-')
+      ranges[i] = paste(r[w[i]], r[(w[i + 1] - 1)], sep = ' to ')
     }
-    ranges[i + 1] = paste(r[w[i + 1]], r[length(r)], sep = '-')
-    paste(unlist(ranges), collapse=' ')
+    ranges[i + 1] = paste(r[w[i + 1]], r[length(r)], sep = ' to ')
+    unlist(ranges)
   }
-
-  SEM = sd(test)*sqrt(1-reliability)
-  if (is.null(roll.length)) roll.length = round(SEM*2+1) # roll.length=NA
-  if (roll.length < 1) stop('Provided roll.length is smaller than 1.')
-  if (is.even(roll.length)){
-    stop(paste('Calculated or provided roll.length is even. Please define roll.length as',
-                 roll.length-1, 'or', roll.length+1))
+  
+  if ((is.null(reliability) & is.null(roll.length))) {
+    roll.length=1
+    reliability=1
   }
-
-  # roll.length=1
-  if (is.numeric(test)) test = factor(test, ordered=TRUE, levels = min(test):max(test)) # is.ordered(test)
+  
+  if (!is.null(reliability)) {
+    if (is.factor(test)) {
+      SEM = sd(as.numeric(as.character(test))) * sqrt(1 - reliability)
+    } else
+      SEM = sd(test) * sqrt(1 - reliability)
+    if (is.null(roll.length)) roll.length = round(SEM*2+1) # roll.length=NA
+  }
+  
+  if (is.null(roll.length)) roll.length = 1
+  if (is.even(roll.length)){roll.length = roll.length-1}
+  if (roll.length <= 1) {
+    roll.length=1
+    warning("roll.length is set to 1. No smoothing applied.") 
+  }
+  
+  # include test scores with zero counts
+  if (is.numeric(test)) test = factor(test, ordered=TRUE,
+                                      levels = min(test):max(test)) 
   if (is.numeric(ref)) ref = factor(ref, ordered=T, levels = min(ref):max(ref))
   tt = table(ref, test)
-  ts.npv = rollsum(tt[1,], roll.length, fill=NA)/(rollsum(tt[2,]+tt[1,],roll.length, fill=NA))
+  ts.npv = rollsum(tt[1,], roll.length, fill=NA)/
+    (rollsum(tt[2,]+tt[1,],roll.length, fill=NA))
   ts.ppv = 1- ts.npv
   st0 = sum(tt[1,])
   st1 = sum(tt[2,])
   sample.prevalence = st1/(st0+st1)
 
   if (is.null(pretest.prob)) pretest.prob = sample.prevalence
-  ts.ilr = (rollsum(tt[2,], roll.length, fill=NA)*st0)/(rollsum(tt[1,],roll.length, fill=NA)*st1)
-  #
+  ts.ilr = (rollsum(tt[2,], roll.length, fill=NA)*st0)/
+    (rollsum(tt[1,],roll.length, fill=NA)*st1)
   ts.sppv =  (rollsum(tt[2,], roll.length, fill=NA)/st1)/
-    ((rollsum(tt[2,], roll.length, fill=NA)/st1)+(rollsum(tt[1,],roll.length, fill=NA)/st0))
+    ((rollsum(tt[2,], roll.length, fill=NA)/st1)+
+       (rollsum(tt[1,],roll.length, fill=NA)/st0))
   ts.snpv =  (rollsum(tt[1,], roll.length, fill=NA)/st0)/
-    ((rollsum(tt[1,], roll.length, fill=NA)/st0)+(rollsum(tt[2,],roll.length, fill=NA)/st1))
+    ((rollsum(tt[1,], roll.length, fill=NA)/st0)+
+       (rollsum(tt[2,],roll.length, fill=NA)/st1))
 
   if (is.null(preselected.thresholds)){
     decuse = paste('Decision use = ', decision.use, '.', sep='')
@@ -301,8 +333,10 @@ RPV <- function(ref, test, pretest.prob = NULL, reliability, roll.length = NULL,
     decuse = 'Decision use = preselected.thresholds.'
   }
   if (extend & any(is.na(ts.npv))){
-    ext = rbind(paste('Reliable Predictive Values for scores ', paste(names(which(is.na(ts.npv))), collapse=' '), ' have been extended.'),
-          unname(decuse))
+    ext = rbind(paste('Reliable Predictive Values for scores ', 
+                      paste(names(which(is.na(ts.npv))), collapse=' '), 
+                      ' have been extended.'), 
+                unname(decuse))
     ts2.npv=na.fill(ts.npv, fill=c('extend', NA, 'extend'))
     ts2.ppv=na.fill(ts.ppv, fill=c('extend', NA, 'extend'))
     ts2.snpv=na.fill(ts.snpv, fill=c('extend', NA, 'extend'))
@@ -321,7 +355,6 @@ RPV <- function(ref, test, pretest.prob = NULL, reliability, roll.length = NULL,
   posttest.prob = ifelse (is.infinite(posttest.odds) & posttest.odds > 0, 1,
       posttest.odds/(posttest.odds+1))
 
-
   rel.conf.level = pnorm((roll.length-1)/2, 0, SEM) - pnorm(-(roll.length-1)/2, 0, SEM)
 
   if (decision.use == 'LR') {
@@ -335,12 +368,14 @@ RPV <- function(ref, test, pretest.prob = NULL, reliability, roll.length = NULL,
                  rel.conf.level=rel.conf.level, decision.odds=decision.odds,
                  limit=limit), digits)) # as.numeric(arg1[1])*2
 
-  scores = 1:ncol(tt); names(scores)=colnames(tt)
+  ttcols = 1:ncol(tt); 
+  names(ttcols)=colnames(tt) # levels(test)
+  
   if (is.null(preselected.thresholds)){
     if (decision.use == 'predictive.value'){
       pd = which(ts2.ppv > limit)
       if (decision.odds < 1){
-        nd = scores[-pd]
+        nd = ttcols[-pd]
         ud = NA
       } else {
         nd = which(ts2.npv > limit)
@@ -349,7 +384,7 @@ RPV <- function(ref, test, pretest.prob = NULL, reliability, roll.length = NULL,
     } else if (decision.use == 'posttest.probability'){
       pd = which(posttest.prob > limit)
       if (decision.odds < 1){
-        nd = scores[-pd]
+        nd = ttcols[-pd]
         ud = NA
       } else {
         nd = which((1-posttest.prob) > limit)
@@ -357,17 +392,17 @@ RPV <- function(ref, test, pretest.prob = NULL, reliability, roll.length = NULL,
     } else if (decision.use == 'standardized.pv'){
       if (decision.odds < 1){
         pd = which(ts2.sppv > limit)
-        nd = scores[-pd]
+        nd = ttcols[-pd]
         ud = NA
       } else {
         pd = which(ts2.sppv > limit)
-        nd = which(ts2.snpv > limit)
+        nd =  
         ud = which(ts2.snpv <= limit & (ts2.sppv <= limit))
       }
     } else if (decision.use == 'LR'){
       if (decision.odds < 1){
         pd = which(ts2.ilr > limit)
-        nd = scores[-pd]
+        nd = ttcols[-pd]
         ud = NA
       } else {
         pd = which(ts2.ilr > limit)
@@ -375,22 +410,36 @@ RPV <- function(ref, test, pretest.prob = NULL, reliability, roll.length = NULL,
         ud = which(ts2.ilr <= limit & (ts2.ilr >= 1/limit))
       }
     }
-    } else { # preselected.thresholds = c(25.2,25.1)
-      numlevels = as.numeric(levels(test))
+  } else { # preselected.thresholds = c(25.2,25.1)
+      numlevels = as.numeric(levels(test)) 
       if (preselected.thresholds[2] > preselected.thresholds[1]){ # positive decisions for higher scores
         pd = which(numlevels > preselected.thresholds[2])
         nd = which(numlevels < preselected.thresholds[1])
-        ud =  which(numlevels >= preselected.thresholds[1]  & (as.numeric(levels(test)) < preselected.thresholds[2]))
+        ud = which(numlevels >= preselected.thresholds[1]  & (as.numeric(levels(test)) < preselected.thresholds[2]))
       } else { # positive decisions for lower scores
         pd = which(numlevels < preselected.thresholds[2])
         nd = which(numlevels > preselected.thresholds[1])
-        ud =  which(numlevels >= preselected.thresholds[2]  & (as.numeric(levels(test)) <= preselected.thresholds[1]))
+        ud = which(numlevels >= preselected.thresholds[2]  & (as.numeric(levels(test)) <= preselected.thresholds[1]))
       }
+      
       names(pd)=colnames(tt)[pd]
       names(nd)=colnames(tt)[nd]
       names(ud)=colnames(tt)[ud]
-    }
-
+  }
+  
+  TN = sum(tt[1, nd]) # tt[1,nd]
+  FN = sum(tt[2, nd])
+  u0 = sum(tt[1, ud])
+  u1 = sum(tt[2, ud])
+  FP = sum(tt[1, pd])
+  TP = sum(tt[2, pd])
+  
+  if (show.table) {
+    showtable = addmargins(matrix(c(TN,FN,u0,u1,FP,TP), ncol=2, byrow=T,
+                          dimnames=list(c('Negative decisions','Uncertain',
+                                          'Positive decisions'),c("0","1"))))
+  }
+ 
   pv = as.matrix(rbind(rnpv=round(ts2.npv, digits), rppv = round(ts2.ppv, digits),
                       rsnpv=round(ts2.snpv, digits), rsppv=round(ts2.sppv, digits),
                       rilr=round(ts2.ilr,digits), rpt.odds = round(posttest.odds,digits), rpt.prob = round(posttest.prob, digits)))
@@ -398,22 +447,44 @@ RPV <- function(ref, test, pretest.prob = NULL, reliability, roll.length = NULL,
 
   col.title = c('Negative Decisions', 'Uncertain', 'Positive Decisions')
   # scores = c(paste(names(nd), collapse = " "), paste(names(ud),collapse = " "), paste(names(pd), collapse = " "))
-  scores = c(mktxtseq(as.numeric(names(nd))),
-             mktxtseq(as.numeric(names(ud))),
-             mktxtseq(as.numeric(names(pd))))
-  n = c(sum(tt[,nd]), sum(tt[,ud]), sum(tt[,pd]))
-  total.sample =  percent(c(sum(tt[,nd])/sum(tt), sum(tt[,ud])/sum(tt),sum(tt[,pd])/sum(tt)), 1)
-  correct.decisions = percent(c(sum(tt[1,nd])/sum(tt[,nd]), NA, sum(tt[2,pd])/sum(tt[,pd])), 1)
-  tns = c(sum(tt[1,nd])/sum(tt[1,]),sum(tt[1,ud])/sum(tt[1, ]), sum(tt[1,pd])/sum(tt[1,]))
-  tps = c(sum(tt[2,nd])/sum(tt[2,]),sum(tt[2,ud])/sum(tt[2, ]), sum(tt[2,pd])/sum(tt[2,]))
-  true.neg.status = percent(tns, 1)
-  true.pos.status = percent(tps, 1)
-  ud.odds = sum(tt[2,ud])/sum(tt[1,ud])
-  realized.odds = round(c(sum(tt[1,nd])/sum(tt[2,nd]), ud.odds, sum(tt[2,pd])/sum(tt[1,pd])), digits)
-  tt2 = as.table(rbind(scores, n, total.sample, correct.decisions, true.neg.status, true.pos.status, realized.odds))
+  # paste(unlist(ranges), collapse=' ')
+  ndr = paste(mktxtseq(as.numeric(names(nd))), collapse=' ')
+  temp =  mktxtseq(as.numeric(names(ud)))
+  if (length(temp) > 1) thresholds.UI = NA else {
+    thresholds.UI=c(L=min(as.numeric(names(ud))),U=max(as.numeric(names(ud))))
+  }
+  udr = paste(temp, collapse=' ')
+  pdr = paste(mktxtseq(as.numeric(names(pd))), collapse=' ')
+  ranges = c(ndr, udr, pdr)
+  names(ranges) = col.title
+  n.nd = sum(tt[,nd])
+  n.ud = sum(tt[,ud])
+  n.pd = sum(tt[,pd])
+  n.tot = sum(tt)
+  n = c(n.nd, n.ud, n.pd)
+  total.sample =  c(n.nd/n.tot, n.ud/n.tot, n.pd/n.tot)
+  correct.decisions = c(TN/n.nd, NA, TP/n.pd)
+  sum.tneg = sum(tt[1,])
+  sum.tpos = sum(tt[2,])
+  tns = c(TN/sum.tneg, sum(tt[1,ud])/sum.tneg, FP/sum.tneg)
+  tps = c(FN/sum.tpos, sum(tt[2,ud])/sum.tpos, TP/sum.tpos)
+  true.neg.status = tns
+  true.pos.status = tps
+  ud.odds = sum(tt[2,ud])/sum(tt[1,ud]) # digits=7
+  realized.odds = c(TN/FN, ud.odds, TP/FP)
+  tt2 = as.matrix(rbind(n, total.sample, correct.decisions, 
+                         true.neg.status, true.pos.status, realized.odds))
   colnames(tt2) = col.title
-
-  out = list(parameters = arg1, messages = ext, rel.pred.values=pv, result=tt2)
+  if (use.perc){
+    tt2[2:5,1] = percent(as.numeric(tt2[2:5,1]),digits) # digits = 1
+    tt2[c(2,4,5),2] = percent(as.numeric(tt2[c(2,4,5),2]),digits)
+    tt2[2:5,3] = percent(as.numeric(tt2[2:5,3]),digits)
+    tt2[6,] = round(as.numeric(tt2[6,]),digits)
+  } 
+  
+  out = list(parameters = arg1, messages = ext, rel.pred.values=pv, 
+         thresholds.UI=thresholds.UI, ranges=ranges, results=data.frame(tt2))
+  if (show.table) out$table = showtable
   return(out)
 }
 
